%--------------------------------------------------------------------------
% Simulation of self-assembly of magnetic nanocubes
% Verner Hkosen 2015
% The code was developed using MATLAB R2014a
%--------------------------------------------------------------------------
%%
clear all
close all
clc

%% PARAMETERS

n=1000000; %number of steps
u0=100000; %making plots each u0 step
particleNum=100; %number of particles
tau=0.000000005; %time step in sec
size=16.16*10^(-9); %total size of the MOVING particle in m
evapRate0=10^7; %nm/s decrease of the z-dimension
time=0; %s


% Dimensions
dim=[150,150,600]; %[x,y,z] volume dimensions in nm
dim_nc=12; %dimension of nanocubes in nm
surfactant=2.08; %thickness of surfactant layer in nm
dim_nc_tot=dim_nc+2*surfactant; %total dimension in nm


% Experimental parameters
c_oa=200; %L/mL oleic acid concentration (c_oa=0 in hexane and c_oa=1000 in oleic acid)
vol_frac=c_oa*10^(-3); %volume fraction of oleic acid 
am_oa=vol_frac*dim(3); %effective hight of oleic acid
%concentration: 9 mg/mL


% Brownian motion
k=1.3806488*10^(-23); %J/K Boltzmann constant
T=298; %temperature in Kelvin
eta_hex=0.30*10^(-3); % %dynamic viscosity of hexane in Pa*s
eta_oa=27.6*10^(-3); %dynamic viscosity of oleic acid in Pa*s
eta=eta_hex*(1-vol_frac)+eta_oa*vol_frac; %effective dynamic viscosity used in the iterations

C_O=(size^3*3/(4*pi))^(1/3); %Equal volume sphere radius
C_cube=1.06*C_O;

f=6*(pi)*eta*C_cube; %translational friction coefficient
fr=8*(pi)*eta*C_cube^3; %rotational friction coefficient

D=k*T/f; %m^2/s initial translational diffusion coefficient given by the Einstein relation
Dr=k*T/fr; %m^2/s initial rotational diffusion coefficient given by the Einstein relation

step=sqrt(2*tau*D)*10^9; %initial step size in nm given by the 1D mean square displacement
stepr=sqrt(2*tau*Dr); %initial angular step size in radians given by the "1D" mean square angular displacement

D_red=k*T/f*eta; %reduced translational diffusion coefficient (wrt. viscosity)


% Magnetism
mu0=4*pi*10^(-7); %N/A^2 vacuum permeability

V=(12*10^(-9))^3; %m^3 volume
Ms=4.8*10^(5)*0.9; %A/m saturation magnetization (0.9 correction factor for disordered surface)
%ms=1.174*10^(-18); %Am^2 intrinsic magnetization (not calculated, from paper)
ms=Ms*V; %Am^2 intrinsic magnetization (calculated)

M=ones(particleNum,3)*sqrt(1/3); %magnetic moment unit vectors
Mu=M; %magnetic moment unit vector with respect to the cube coordinate system

Kd=ms^2*mu0/(4*pi*10^(-27)); %J/nm^3 constant

H0=1200; %G magnetic field
H_grad=1;%*10^(-5); %G/nm magnetic field gradient
H=[0 0 1]; %magnetic field unit vector

Kz=@(x) -ms*(H0-H_grad*x)*10^(-4); %J constant
Kh=ms*mu0/(4*pi*10^(-27)); %T*nm^3 magetic field constant

Ka=-3.784*4184/(6.022*10^(23)); %J anisotropy constant


% tau_0=10^(-9); %attempt time


%van der Waals interactions
alfaT_hex=0.426; %product of thermal expansion coefficient and temp. for hexane 
alfaT_oa=0.0123; %product of thermal expansion coefficient and temp. for oleic acid
A_h_mix=3/4*k*T*(3/(alfaT_hex*(1-vol_frac)+alfaT_oa*vol_frac)+7); %J Hamaker constant of the solvent
A_h_mag=12.6*10^(-20); %J Hamaker constant of Fe3O4

A_h=(sqrt(A_h_mag)-sqrt(A_h_mix))^2; %Hamaker constant of Fe3O4 in dispersion


% Steric repulsion
Vs_hex=2.018*10^(-28); %m^3
Vs_oa=5.240*10^(-28); %m^3
Vs=Vs_hex*(1-vol_frac)+Vs_oa*vol_frac;
phi=1;
chi_hex=0.364;
chi_oa=0.340;
chi=chi_hex*(1-vol_frac)+chi_oa*vol_frac;
nu=3.57*10^18; %m^-2

K_mix1=pi*dim_nc*10^(-9)/(2*Vs)*(1/2-chi)*10^(-18)*k*T; %J/nm^2
K_mix2=pi*dim_nc*10^(-9)/Vs*(1/2-chi)*dim_nc^2*10^(-18)*k*T; %J
K_elastic=pi*nu*dim_nc*10^(-18)*k*T; %J

K_steric=[K_mix1,K_mix2,K_elastic];


%% Figures
% F=struct('cdata',[],'colormap',[]);
frameCount=1;
h=[];
hi=[];
mean_std=[0 0];

%% BROWNIAN MOTION

% Placing particles out at random sites in the defined space

theta=2*pi*rand(particleNum,3); %angluar orientation for each particle in the x-y plane
pos=[(dim(1)-dim_nc_tot*sqrt(3))*rand(particleNum,1),(dim(2)-dim_nc_tot*sqrt(3))*rand(particleNum,1),(dim(3)-dim_nc_tot*sqrt(3))*rand(particleNum,1)]+dim_nc_tot*sqrt(3)/2;


% Making sure that no particles overlap
c=2;
i=1;
while i<particleNum

    for j=c:particleNum
         
      if pos(j,:)>=(pos(i,:)-dim_nc_tot) & pos(j,:)<=(pos(i,:)+dim_nc_tot)
           
        while pos(j,:)>=(pos(i,:)-dim_nc_tot) & pos(j,:)<=(pos(i,:)+dim_nc_tot)
            pos(j,:)=[(dim(1)-dim_nc_tot*sqrt(3))*rand,(dim(2)-dim_nc_tot*sqrt(3))*rand,(dim(3)-dim_nc_tot*sqrt(3))*rand]+dim_nc_tot*sqrt(3)/2;
        end
        
        i=0;
        c=1;
        break

      end
      
    end
   
  i=i+1;
  c=c+1;    
end

movement=zeros(particleNum,3);
rotation=movement;

[pos,theta,M,Mu]=magnetic(particleNum,dim_nc,surfactant,dim,A_h,pos,movement,theta,rotation,Kd,Ka,Kz,K_steric,M,Mu,H,T); %function

% The "validate" matrix tell the relative position of the particle with
% respect to each other. The first column exhibits a value 1 or 0 (true or
% false) if particle 2 is on the right side of particle 1. The second
% column tells, in the same manner, if particle 2 is above (true) or below
% (false) particle 1. The rows shows the relatie positions between two
% particles. This matrix is returned by the "relativePosition" function.

        
    r_hist=[];

    for y=1:particleNum-1
       
       r=pos(y+1:end,:);
       r(:,1)=r(:,1)-pos(y,1);
       r(:,2)=r(:,2)-pos(y,2);
       r(:,3)=r(:,3)-pos(y,3);
       
       r_hist=[r_hist;r];
        
    end
    
    clear r
    r_hist=sqrt(r_hist(:,1).^2+r_hist(:,2).^2+r_hist(:,3).^2);
    
    
     %Plotting cubes at initial positions
        h(frameCount)=figure;
        hold off

            for i=1:particleNum

                cubePlot(i,dim_nc,dim_nc_tot,pos,theta,M,Mu) %function

            end

        axis equal
        axis([0 dim(1) 0 dim(2) 0 dim(3)]);
        title(['Elapsed time: ',num2str(time,'%.4f'),' \mus']);
        xlabel('x [nm]');
        ylabel('y [nm]');
        zlabel('z [nm]');
        view(3,4) 
        box on
        
        drawnow     %makes movie frames
%         F(frameCount) = getframe(gcf);
        
        %Polotting histogram of pair distances
        hold off
        hi(frameCount)=figure;
        hist(r_hist);
        title(['Elapsed time: ',num2str(time,'%.4f'),' \mus']);
        xlabel('Distance between pairs [nm]');
        ylabel('Population');
        mean_std=[mean(r_hist) std(r_hist)];


 %% n steps  
        
for u=1:n
    
%Correction of physical parameters upon evaporation

K_steric(1:2)=K_steric(1:2)*Vs/(0.5-chi);
    
evapRate=evapRate0*1/(1+exp(18.8*(vol_frac-0.45)));
dim(3)=dim(3)-evapRate*tau;
vol_frac=am_oa/dim(3); %new volume fraction; dependent terms parameters below

Vs=Vs_hex*(1-vol_frac)+Vs_oa*vol_frac;
chi=chi_hex*(1-vol_frac)+chi_oa*vol_frac;
K_steric(1:2)=K_steric(1:2)*(0.5-chi)/Vs;

time=time+tau*10^(6); %s
    
eta=eta_hex*(1-vol_frac)+eta_oa*vol_frac; %weightned average

A_h_mix=3/4*k*T*(3/(alfaT_hex*(1-vol_frac)+alfaT_oa*vol_frac)+7); %Hamaker constant of the solvent
A_h=(sqrt(A_h_mag)-sqrt(A_h_mix))^2; %Hamaker constant of Fe3O4 in dispersion


D=D_red/eta;
tau=0.5*(step*10^(-9))^2/D; %time of each step in u=u+1




% Relative position between two particles 

validate=[];
  
    for j=1:particleNum-1
       
       r=pos(j+1:end,:);
       r(:,1)=r(:,1)-pos(j,1);
       r(:,2)=r(:,2)-pos(j,2);
       r(:,3)=r(:,3)-pos(j,3);
       
       validate=[validate;r];
        
    end

clear r
    
[row col]=find(validate==0);

if isempty(row)==0
   le=length(row);
    for o=1:le
        if rand<0.5
            validate(row(o),col(o))=1;
        else
            validate(row(o),col(o))=-1;
        end
    end
    
    
end

validate=validate./abs(validate);


    
% Moving the particles one step. The step for each dimension for each
% particle are indicated by the "movement" matrix where the row entries
% exhibit the step size for each paricle, with the first column being the
% x-coordinate, the second the y-coordinate and the third the z-coordinate.

 movement=step*randn(particleNum,3);
 rotation=stepr*randn(particleNum,3);
 

% Potentials

[pos,theta,M,Mu]=magnetic(particleNum,dim_nc,surfactant,dim,A_h,pos,movement,theta,rotation,Kd,Ka,Kz,K_steric,M,Mu,H,T); %function


% Steric repulsion

    r_dist=[];

    for y=1:particleNum-1
       
       r=pos(y+1:end,:);
       r(:,1)=r(:,1)-pos(y,1);
       r(:,2)=r(:,2)-pos(y,2);
       r(:,3)=r(:,3)-pos(y,3);
       r(:,4)=y;
       r(:,5)=[y+1:particleNum]';
       
       r_dist=[r_dist;r];
        
    end
    
    r_dist=[sqrt(r_dist(:,1).^2+r_dist(:,2).^2+r_dist(:,3).^2),r_dist(:,4),r_dist(:,5)];
    r_hist=r_dist(:,1);
    r_dist=[r_dist,[1:length(r_dist(:,1))]'];
    L=find(r_dist(:,1)<dim_nc*sqrt(3));
    r_dist=r_dist(L,:);
    
    if isempty(r_dist)==false
        l=length(r_dist(:,1));
        
        for y=1:l
            [pos,theta]=stericRepulsion(r_dist(y,2),r_dist(y,3),dim_nc,movement,validate,r_dist(y,4),pos,theta); %function
        end
   
    end
    
 
%Plotting the nanocubes

    if rem(u,u0)==false
        
        u
        time
        frameCount=frameCount+1;
        
        h(frameCount)=figure;

        hold off

            for i=1:particleNum

                cubePlot(i,dim_nc,dim_nc_tot,pos,theta,M,Mu) %function

            end

        axis equal
        axis([0 dim(1) 0 dim(2) 0 dim(3)]);
        title(['Elapsed time: ',num2str(time,'%.4f'),' \mus']);
        xlabel('x [nm]');
        ylabel('y [nm]');
        zlabel('z [nm]');
        view(3,4) 
        box on
        
        drawnow     %makes movie frames
%         F(frameCount) = getframe(gcf);
        
        hold off
        hi(frameCount)=figure;
        hist(r_hist);
        title(['Elapsed time: ',num2str(time,'%.4f'),' \mus']);
        xlabel('Distance between pairs [nm]');
        ylabel('Population');
        mean_std=[mean(r_hist) std(r_hist)];
      
    end
 

end

savefig(hi,'histograms_zerofield.fig')
savefig(h,'snapshots_zerofield.fig')
close(h,hi)

dlmwrite('pos.txt',pos)
dlmwrite('theta.txt',theta)
dlmwrite('Mu.txt',Mu)

%movieWriter(F); %writing movie to .avi file
